/*******************************

   Author : Sebastien Verel
   Date   : March, 30th, 2014

   Licence : LGPLv3

   Test program of the L-system suit

*********************************/

public class Main {
	
    public static void testWindow() {
	// Create a windows of size 400x400 pixels
	Window window = new Window(400, 400);

	// Draw a square
	window.draw(100, 100, 100, 200);
	window.draw(100, 200, 200, 200);
	window.draw(200, 200, 200, 100);
	window.draw(200, 100, 100, 100);
    }

    public static void testTurtle() {
	// Create a windows of size 400x400 pixels
	Window window = new Window(400, 400);

	// Create a turtle on the window
	Turtle turtle = new Turtle(window);

	// Put the turtle on the point (200, 300)
	turtle.setPosition(200, 300);

	// Draw a square with a turtle
	turtle.penDown();
	turtle.forward(100);
	turtle.turnRight(90);
	turtle.forward(100);
	turtle.turnRight(90);
	turtle.forward(100);
	turtle.turnRight(90);
	turtle.forward(100);
    }

    /*
      @param syst the L-system
      @param nbIteration number of iteration
     */
    public static void testLsystem(int syst, int nbIteration) {
	Lsystem lsystem = new Lsystem();

	String axiome = "" ;

	switch (syst) {
	case 0:
	    // Quadratic Kock Island 
	    axiome = "F-F-F-F";
	    lsystem.addProduction(new Production('F', "F-F+F+FF-F-F+F"));
	    break;
	case 1:
	    // Kock curve 
	    axiome = "F";
	    lsystem.addProduction(new Production('F', "F-F++F-F"));
	    break;
	case 2:
	    // Sierpinski triangle
	    axiome = "A";
	    lsystem.addProduction(new Production('A', "B+A+B"));
	    lsystem.addProduction(new Production('B', "A-B-A"));
	    break;
	case 3:
	    // Original Algae
	    axiome = "A";
	    lsystem.addProduction(new Production('A', "AB"));	    
	    lsystem.addProduction(new Production('B', "A"));	    
	}

	// Derivation
	String word = new String(axiome);

	for(int i = 0; i < nbIteration; i++)
	    word = lsystem.derive(word);

	// Print the result
	System.out.println("After " + nbIteration +" iterations:");
	System.out.println(word);
    }

    /*
      @param syst the L-system
      @param nbIteration number of iteration
     */
    public static void testTranscription(int syst, int nbIteration) {
	// Create a windows of size 600x600 pixels
	Window window = new Window(600, 600);

	// Create a turtle on the window
	Turtle turtle = new Turtle(window);

	// Put the turtle on point (200, 300)
	turtle.setPosition(300, 500);

	// Transcription into screen
	Transcription transcription = new Transcription(turtle);

	// the L-system
	Lsystem lsystem = new Lsystem();

	// Transcription of the intermediate letters into drawing letters
	Lsystem finalTrans = new Lsystem();

	String axiome = "" ;

	switch (syst) {
	case 0:
	    // Quadratic Kock Island 
	    axiome = "F-F-F-F";
	    lsystem.addProduction(new Production('F', "F-F+F+FF-F-F+F"));
	    // Set the length of a "forward"
	    transcription.setLength(3);
	    // Set the angle of a "turn"
	    transcription.setAngle(90);
	    break;
	case 1:
	    // Kock curve 
	    axiome = "F";
	    lsystem.addProduction(new Production('F', "F+F-F-F+F"));
	    // Set the length of a "forward"
	    transcription.setLength(5);
	    // Set the angle of a "turn"
	    transcription.setAngle(90);
	    break;
	case 2:
	    // Sierpinski triangle
	    axiome = "A";
	    lsystem.addProduction(new Production('A', "B+A+B"));
	    lsystem.addProduction(new Production('B', "A-B-A"));
	    // Set the length of a "forward"
	    transcription.setLength(5);
	    // Set the angle of a "turn"
	    transcription.setAngle(60);

	    // to translate the A and B into F
	    finalTrans.addProduction(new Production('A', "F"));
	    finalTrans.addProduction(new Production('B', "F"));
	    break;
	case 3:
	    // Original Algae
	    axiome = "A";
	    lsystem.addProduction(new Production('A', "F-[A]++B"));
	    lsystem.addProduction(new Production('B', "FA"));
	    transcription.setLength(60);
	    transcription.setAngle(25);

	    // to translate the A and B into nothing
	    finalTrans.addProduction(new Production('A', ""));
	    finalTrans.addProduction(new Production('B', ""));
	    break;
	case 4:
	    // Fractal plant
	    axiome = "X";
	    lsystem.addProduction(new Production('X', "F-[[X]+X]+F[+FX]-X"));
	    lsystem.addProduction(new Production('F', "FF"));
	    transcription.setLength(6);
	    transcription.setAngle(25);

	    // to translate the A and B into nothing
	    finalTrans.addProduction(new Production('X', ""));
	}

	String word = new String(axiome);

	for(int i = 0; i < nbIteration; i++)
	    word = lsystem.derive(word);	

	word = finalTrans.derive(word);

	// Draw the word
	transcription.draw(word);
    }


    /***
     *
     *    The Main
     *
     ***/
    public static void main(String[] args) {
	// test of the graphical part
	testWindow();

	// test of the turtle part
	//testTurtle();

	// test of the L-system
	//testLsystem(0, 3);
	//testLsystem(1, 3);
	//testLsystem(2, 3);
	//testLsystem(3, 3);

	// test of the transcription
	//testTranscription(0, 3);
	//testTranscription(1, 4);
	//testTranscription(2, 5);
	// test of the transcription with [ and ]
	//testTranscription(3, 5);
	//testTranscription(4, 5);
	
    }

}
